@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2009, 2010, 2011 Nicira Networks.
+ * Copyright (c) 2008, 2009, 2010, 2011, 2012 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,7 +16,6 @@
 
 #include <config.h>
 #include "poll-loop.h"
-#include <assert.h>
 #include <errno.h>
 #include <inttypes.h>
 #include <poll.h>
@@ -55,9 +54,10 @@ struct poll_waiter {
 /* All active poll waiters. */
 static struct list waiters = LIST_INITIALIZER(&waiters);
 
-/* Max time to wait in next call to poll_block(), in milliseconds, or -1 to
+/* Time at which to wake up the next call to poll_block(), in milliseconds as
+ * returned by time_msec(), LLONG_MIN to wake up immediately, or LLONG_MAX to
  * wait forever. */
-static int timeout = -1;
+static long long int timeout_when = LLONG_MAX;
 
 /* Location where waiter created. */
 static const char *timeout_where;
@@ -82,16 +82,6 @@ poll_fd_wait(int fd, short int events, const char *where)
     return new_waiter(fd, events, where);
 }
 
-/* The caller must ensure that 'msec' is not negative. */
-static void
-poll_timer_wait__(int msec, const char *where)
-{
-    if (timeout < 0 || msec < timeout) {
-        timeout = msec;
-        timeout_where = where;
-    }
-}
-
 /* Causes the following call to poll_block() to block for no more than 'msec'
  * milliseconds.  If 'msec' is nonpositive, the following call to poll_block()
  * will not block at all.
@@ -105,14 +95,25 @@ poll_timer_wait__(int msec, const char *where)
 void
 poll_timer_wait(long long int msec, const char *where)
 {
-    poll_timer_wait__((msec < 0 ? 0
-                       : msec > INT_MAX ? INT_MAX
-                       : msec),
-                      where);
+    long long int now = time_msec();
+    long long int when;
+
+    if (msec <= 0) {
+        /* Wake up immediately. */
+        when = LLONG_MIN;
+    } else if ((unsigned long long int) now + msec <= LLONG_MAX) {
+        /* Normal case. */
+        when = now + msec;
+    } else {
+        /* now + msec would overflow. */
+        when = LLONG_MAX;
+    }
+
+    poll_timer_wait_until(when, where);
 }
 
 /* Causes the following call to poll_block() to wake up when the current time,
- * as returned by time_msec(), reaches 'msec' or later.  If 'msec' is earlier
+ * as returned by time_msec(), reaches 'when' or later.  If 'when' is earlier
  * than the current time, the following call to poll_block() will not block at
  * all.
  *
@@ -123,13 +124,12 @@ poll_timer_wait(long long int msec, const char *where)
  * Ordinarily the 'where' argument is supplied automatically; see poll-loop.h
  * for more information. */
 void
-poll_timer_wait_until(long long int msec, const char *where)
+poll_timer_wait_until(long long int when, const char *where)
 {
-    long long int now = time_msec();
-    poll_timer_wait__((msec <= now ? 0
-                       : msec < now + INT_MAX ? msec - now
-                       : INT_MAX),
-                      where);
+    if (when < timeout_when) {
+        timeout_when = when;
+        timeout_where = where;
+    }
 }
 
 /* Causes the following call to poll_block() to wake up immediately, without
@@ -156,7 +156,8 @@ poll_immediate_wake(const char *where)
 static void
 log_wakeup(const char *where, const struct pollfd *pollfd, int timeout)
 {
-    static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(120, 120);
+    static struct vlog_rate_limit trace_rl = VLOG_RATE_LIMIT_INIT(1, 1);
+    static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(10, 10);
     enum vlog_level level;
     int cpu_usage;
     struct ds s;
@@ -199,6 +200,11 @@ log_wakeup(const char *where, const struct pollfd *pollfd, int timeout)
     }
     if (cpu_usage >= 0) {
         ds_put_format(&s, " (%d%% CPU usage)", cpu_usage);
+
+        if (!vlog_should_drop(THIS_MODULE, level, &trace_rl)) {
+            ds_put_char(&s, '\n');
+            format_backtraces(&s, 2);
+        }
     }
     VLOG(level, "%s", ds_cstr(&s));
     ds_destroy(&s);
@@ -215,6 +221,7 @@ poll_block(void)
 
     struct poll_waiter *pw, *next;
     int n_waiters, n_pollfds;
+    int elapsed;
     int retval;
 
     /* Register fatal signal events before actually doing any real work for
@@ -236,15 +243,15 @@ poll_block(void)
         n_pollfds++;
     }
 
-    if (!timeout) {
+    if (timeout_when == LLONG_MIN) {
         COVERAGE_INC(poll_zero_timeout);
     }
-    retval = time_poll(pollfds, n_pollfds, timeout);
+    retval = time_poll(pollfds, n_pollfds, timeout_when, &elapsed);
     if (retval < 0) {
         static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(1, 5);
         VLOG_ERR_RL(&rl, "poll: %s", strerror(-retval));
     } else if (!retval) {
-        log_wakeup(timeout_where, NULL, timeout);
+        log_wakeup(timeout_where, NULL, elapsed);
     }
 
     LIST_FOR_EACH_SAFE (pw, next, node, &waiters) {
@@ -254,7 +261,7 @@ poll_block(void)
         poll_cancel(pw);
     }
 
-    timeout = -1;
+    timeout_when = LLONG_MAX;
     timeout_where = NULL;
 
     /* Handle any pending signals before doing anything else. */
@@ -281,7 +288,7 @@ static struct poll_waiter *
 new_waiter(int fd, short int events, const char *where)
 {
     struct poll_waiter *waiter = xzalloc(sizeof *waiter);
-    assert(fd >= 0);
+    ovs_assert(fd >= 0);
     waiter->fd = fd;
     waiter->events = events;
     waiter->where = where;
