@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011 Nicira Networks.
+ * Copyright (c) 2011, 2012 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,7 +18,6 @@
 
 #include "route-table.h"
 
-#include <assert.h>
 #include <arpa/inet.h>
 #include <sys/socket.h>
 #include <linux/rtnetlink.h>
@@ -161,8 +160,8 @@ void
 route_table_register(void)
 {
     if (!register_count) {
-        assert(!nln);
-        assert(!route_notifier);
+        ovs_assert(!nln);
+        ovs_assert(!route_notifier);
 
         nln = nln_create(NETLINK_ROUTE, RTNLGRP_IPV4_ROUTE,
                          (nln_parse_func *) route_table_parse, &rtmsg);
@@ -246,6 +245,7 @@ route_table_reset(void)
     rtmsg->rtgen_family = AF_INET;
 
     nl_dump_start(&dump, rtnl_sock, &request);
+    ofpbuf_uninit(&request);
 
     while (nl_dump_next(&dump, &reply)) {
         struct route_table_msg msg;
@@ -437,6 +437,8 @@ name_table_reset(void)
     rtmsg->rtgen_family = AF_INET;
 
     nl_dump_start(&dump, rtnl_sock, &request);
+    ofpbuf_uninit(&request);
+
     while (nl_dump_next(&dump, &reply)) {
         struct rtnetlink_link_change change;
 
