@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011 Nicira Networks.
+ * Copyright (c) 2011, 2012, 2013 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,37 +22,23 @@
 #include "flow.h"
 #include "odp-util.h"
 #include "ofpbuf.h"
+#include "util.h"
 #include "vlog.h"
 
-int
-main(void)
+static int
+parse_keys(void)
 {
     int exit_code = 0;
     struct ds in;
 
     ds_init(&in);
-    vlog_set_levels_from_string("odp_util:console:dbg");
-    while (!ds_get_line(&in, stdin)) {
+    vlog_set_levels_from_string_assert("odp_util:console:dbg");
+    while (!ds_get_test_line(&in, stdin)) {
         enum odp_key_fitness fitness;
         struct ofpbuf odp_key;
         struct flow flow;
         struct ds out;
         int error;
-        char *s;
-
-        /* Delete comments, skip blank lines. */
-        s = ds_cstr(&in);
-        if (*s == '#') {
-            puts(s);
-            continue;
-        }
-        if (strchr(s, '#')) {
-            *strchr(s, '#') = '\0';
-        }
-        if (s[strspn(s, " ")] == '\0') {
-            putchar('\n');
-            continue;
-        }
 
         /* Convert string to OVS DP key. */
         ofpbuf_init(&odp_key, 0);
@@ -84,7 +70,7 @@ main(void)
         /* Convert cls_rule back to odp_key. */
         ofpbuf_uninit(&odp_key);
         ofpbuf_init(&odp_key, 0);
-        odp_flow_key_from_flow(&odp_key, &flow);
+        odp_flow_key_from_flow(&odp_key, &flow, flow.in_port);
 
         if (odp_key.size > ODPUTIL_FLOW_KEY_BYTES) {
             printf ("too long: %zu > %d\n",
@@ -105,3 +91,49 @@ main(void)
 
     return exit_code;
 }
+
+static int
+parse_actions(void)
+{
+    struct ds in;
+
+    ds_init(&in);
+    vlog_set_levels_from_string_assert("odp_util:console:dbg");
+    while (!ds_get_test_line(&in, stdin)) {
+        struct ofpbuf odp_actions;
+        struct ds out;
+        int error;
+
+        /* Convert string to OVS DP actions. */
+        ofpbuf_init(&odp_actions, 0);
+        error = odp_actions_from_string(ds_cstr(&in), NULL, &odp_actions);
+        if (error) {
+            printf("odp_actions_from_string: error\n");
+            goto next;
+        }
+
+        /* Convert odp_actions back to string. */
+        ds_init(&out);
+        format_odp_actions(&out, odp_actions.data, odp_actions.size);
+        puts(ds_cstr(&out));
+        ds_destroy(&out);
+
+    next:
+        ofpbuf_uninit(&odp_actions);
+    }
+    ds_destroy(&in);
+
+    return 0;
+}
+
+int
+main(int argc, char *argv[])
+{
+    if (argc == 2 &&!strcmp(argv[1], "parse-keys")) {
+        return parse_keys();
+    } else if (argc == 2 && !strcmp(argv[1], "parse-actions")) {
+        return parse_actions();
+    } else {
+        ovs_fatal(0, "usage: %s parse-keys | parse-actions", argv[0]);
+    }
+}
